<?php

namespace JReviewsDeploy\Controllers;

use JReviewsDeploy\Services\AppStoreService;
use JReviews\App\Actions\InstallAddonAction;
use JReviews\App\Actions\InstallCorePackageExtrasAction;
use JReviews\App\Actions\UninstallAddonAction;
use JReviews\App\Actions\ClearCacheAction;
use Plugin_Upgrader;

defined('ABSPATH') or die;

class DeployController 
{
    protected $errors = [];

    protected $appstore;

    public function __construct()
    {
        $this->appstore = new AppStoreService();

        $page = $_GET['page'] ?? null;

        if ($page == 'jreviews-deploy') {
            global $wp_filter;

            unset($wp_filter['admin_notices']);
        }

        add_action('admin_enqueue_scripts', fn() => wp_enqueue_script('htmx', 'https://unpkg.com/htmx.org@1.9.12', array(), null, true));
    
        add_action('wp_ajax_jreviewsdeploy.startUpdate', fn() => $this->startUpdate());

        add_action('admin_menu', function() {
            $icon = 'data:image/svg+xml;base64,'.base64_encode('<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" fill="currentColor" class="fwd-w-5 fwd-h-5">
            <path fill-rule="evenodd" d="M9.315 7.584C12.195 3.883 16.695 1.5 21.75 1.5a.75.75 0 0 1 .75.75c0 5.056-2.383 9.555-6.084 12.436A6.75 6.75 0 0 1 9.75 22.5a.75.75 0 0 1-.75-.75v-4.131A15.838 15.838 0 0 1 6.382 15H2.25a.75.75 0 0 1-.75-.75 6.75 6.75 0 0 1 7.815-6.666ZM15 6.75a2.25 2.25 0 1 0 0 4.5 2.25 2.25 0 0 0 0-4.5Z" clip-rule="evenodd" />
            <path d="M5.26 17.242a.75.75 0 1 0-.897-1.203 5.243 5.243 0 0 0-2.05 5.022.75.75 0 0 0 .625.627 5.243 5.243 0 0 0 5.022-2.051.75.75 0 1 0-1.202-.897 3.744 3.744 0 0 1-3.008 1.51c0-1.23.592-2.323 1.51-3.008Z" />
          </svg>');

            add_menu_page(
                'JReviews Deploy',
                'JReviewsDeploy',
                'manage_options',
                'jreviews-deploy',
                fn() => $this->deploy(),
                $icon,
                51
            );
        });
    }

    protected function deploy()
    {
        $this->checkLicense();

        $this->checkRequirements();
        
        $content = $this->render('default');

        include(__DIR__ . '/../../templates/layout.php');
    }

    protected function render($template)
    {
        ob_start();
        include(__DIR__ . "/../../templates/$template.php");
        return ob_get_clean(); 
    }

    protected function selfUpdateCheck($transient) 
    {
        if (empty($transient->checked)) {
            return $transient;
        }
    
        $plugin = 'jreviewsdeploy/jreviewsdeploy.php';
    
        if (isset($transient->checked[$plugin])) {
            $response = $this->appstore->selfUpdateCheck();
            $transient->response[$plugin] = $response;
        }
    
        return $transient;
    }    

    protected function startUpdate()
    {
        $products = $this->appstore->installedProducts();

        $updatable = array_filter($products, function($product) {
            if ($product['new']) {
                return true;
            }

            return false;
        });

        if (empty($updatable)) {
            wp_die($this->render('default'));
        }

        $product = reset($updatable);

        try {

            if (in_array($product['name'], ['jreviews', 's2framework'])) {
                $result = $this->installPlugin($product['name']);
            } else {
                $result = $this->installAddon($product['name']);
            }

            if ( ! $result) {
                $this->logError('Error installing '.$product['name']);
            }

        } catch (\Exception $e) {

            $this->logError($e->getMessage());
        
        }

        wp_die($this->render('default'));
    }

    protected function checkLicense()
    {
        $licenseKey = \JReviewsDeploy\jr_license();

        if (! $licenseKey) {
            wp_die($this->render('missing_license'));
        }

        try {
            if (! $this->appstore->validateLicense()) {
                wp_die($this->render('invalid_license'));
            }
        } catch (\Exception $e) {
            wp_die($this->render('connection_error'));
        }    
    }

    protected function checkRequirements()
    {
        if (version_compare(PHP_VERSION, '8.1', '<')) {
            wp_die($this->render('failed_requirements'));
        }
    }

    protected function installPlugin($name)
    {
        try {
            
            $path = $this->appstore->downloadPluginPackage($name);

            if (! $path) {
                return false;
            }

        } catch (\Exception $e) {

            throw new \Exception("Error downloading $name: ".$e->getMessage(), 500);
        }

        require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php'; 
        
        $plugin_upgrader = new Plugin_Upgrader();

		$result = $plugin_upgrader->run([
            'package'           => $path,
            'destination'       => WP_PLUGIN_DIR,
            'clear_destination' => true,
            'clear_working'     => true,
            'hook_extra'        => [
                'type'   => 'plugin',
                'action' => 'install',
            ],
        ]);
    
        unlink($path);

        if (is_wp_error($result)) {
            // Error handling if installation fails
            $this->logError("Error installing $name: ".$result->get_error_message());
            return false;
        }

        $result = activate_plugin(plugin: WP_PLUGIN_DIR."/$name/$name.php", redirect: false, silent: true);

        if (is_wp_error($result)) {
            $this->logError("Error activating $name: ".$result->get_error_message());
            return false;
        }

        if (class_exists(InstallCorePackageExtrasAction::class)) {
            InstallCorePackageExtrasAction::run($name);
        }

        if (class_exists(ClearCacheAction::class)) {
            ClearCacheAction::run();
        }

        return true;
    }

    protected function installAddon($name)
    {
        try {
            
            $path = $this->appstore->downloadAddon($name);
                    
            if (! $path) {
                return false;
            }

            UninstallAddonAction::run($name);

            $disabledAddons = json_decode(wp_unslash(sanitize_text_field($_REQUEST['disabled_addons'])), true);

            $enable = ! in_array($name, $disabledAddons) ? true : false;

            InstallAddonAction::run(name: $name, sourceFullPath: $path, enable: $enable);

            return true;
    
        } catch (\Exception) {

            throw new \Exception("Error downloading $name", 500);
        }
    }

    public function logError(string $message)
    {
        $this->errors[] = $message;
    }

    protected function errors()
    {
        return $this->errors;
    }

    protected function products()
    {
        return $this->appstore->installedProducts();
    }

    protected function getLicenseNumber()
    {
        return \JReviewsDeploy\jr_license();
    }

    protected function endpoint($task)
    {
        return site_url("/wp-admin/admin-ajax.php?action=jreviewsdeploy.$task");
    }

    protected function loadingUrl()
    {
        return site_url('/wp-content/plugins/jreviewsdeploy/templates/loading.svg');
    }

    protected function thereAreProductsWithAvailableUpdates()
    {
        $products = $this->appstore->installedProducts();

        $updatable = array_filter($products, function($product) {
            if ($product['new']) {
                return true;
            }

            return false;
        });

        return count($updatable) > 0;
    }    

    protected function disabledAddons()
    {
        global $wpdb;
    
        $query = $wpdb->prepare("SELECT name FROM {$wpdb->prefix}jreviews_addons WHERE state = %d", 0);

        return $wpdb->get_col($query);
    }  
}