<?php

namespace JReviewsDeploy\Services;

defined('ABSPATH') or die;

class AppStoreService
{
    const VALIDATE_LICENSE_ENDPOINT = 'appstore/validateLicense';

    const VERSIONS_ENDPOINT = 'appstore/versions';

    const DOWNLOAD_ENDPOINT = 'appstore/download';
    
    const DOWNLOAD_COMPONENT_PACKAGE_ENDPOINT = 'appstore/downloadComponentPackage';

    protected ?string $apiKey;

    protected string $host;
    
    protected static $cache = [];

    public function __construct()
    {
        $this->apiKey = \JReviewsDeploy\jr_license();

        $this->host = $_SERVER['HTTP_HOST'] ?? '';
    }

    public function validateLicense()
    {
        if (static::$cache['validateLicense'] ?? null) {
            return static::$cache['validateLicense'];
        }

        $response = $this->connect(self::VALIDATE_LICENSE_ENDPOINT, [
            'license_key' => $this->apiKey,
        ]);

        static::$cache['validateLicense'] = $response['response']['code'] == 200;

        return static::$cache['validateLicense'];        
    }

    public function downloadPluginPackage($name, $destination = null)
    {
        $response = $this->connect(self::DOWNLOAD_COMPONENT_PACKAGE_ENDPOINT, [
            'license_key' => $this->apiKey,
            'product'=> $name
        ]);

        if ($response['response']['code'] !== 200) {
            return false;
        }

        if (! $destination) {

            $filepath = get_temp_dir().uniqid($name.'-').'.zip';
        
        } else {
        
            $filepath = $destination;

            if (! file_exists($filepath)) {
                mkdir($filepath, 0755);
            }

            $filepath = $filepath.'/'.uniqid($name.'-').'.zip';
        }

        file_put_contents($filepath, $response['body']);
        
        return $filepath;
    }

    public function downloadAddon($name, $destination = null)
    {
        $filepath = null;

        $response = $this->connect(self::DOWNLOAD_ENDPOINT, [
            'license_key' => $this->apiKey,
            'product'=> $name
        ]);

        if ($response['response']['code'] !== 200) {
            return false;
        }

        if (! $destination) {

            $filepath = get_temp_dir().uniqid($name.'-').'.zip';
        
        } else {
        
            $filepath = $destination;

            if (! file_exists($filepath)) {
                mkdir($filepath, 0755);
            }

            $filepath = $filepath.'/'.uniqid($name.'-').'.zip';
        }

        file_put_contents($filepath, $response['body']);

        return $filepath;
    }

    public function latestVersions()
    {
        if (static::$cache['latestVersions'] ?? null) {
            return static::$cache['latestVersions'];
        }

        $response = $this->connect(self::VERSIONS_ENDPOINT);

        if ($response['response']['code'] !== 200) {
            return false;
        }

        static::$cache['latestVersions'] = json_decode($response['body'], true);

        return static::$cache['latestVersions'];
    }    

    public function installedProducts()
    {
        $installed = array_merge([
            (object) [
                'name' => 's2framework',
                'title' => 's2framework',
                'version' => \JReviewsDeploy\plugin_version('s2framework')
            ],
            (object) [
                'name' => 'jreviews',
                'title' => 'JReviews',
                'version' => \JReviewsDeploy\plugin_version('jreviews')
            ],
        ], \JReviewsDeploy\jr_addons());

        $available = $this->latestVersions();

        $products = [];

        foreach ($installed as $i) {
            if (! isset($available[$i->name])) {
                continue;
            }

            if ($i->version == '1.0.5' || version_compare($i->version ?? 0, $available[$i->name], '<')) {
                $products[] = [
                    'title' => $i->title,
                    'name' => $i->name,
                    'current' => $i->version,
                    'new' => $available[$i->name]
                ];
            } else {
                $products[] = [
                    'title' => $i->title,
                    'name' => $i->name,
                    'current' => $i->version,
                    'new' => null,
                ];
            }
        }

        return $products;
    }
        
    protected function connect($endpoint, $payload = [])
    {
        $endpoint = JREVIEWSDEPLOY_BACKOFFICE_API_URL . "/$endpoint";
    
        $installedVersions = $this->installedVersions();
    
        $payload = array_merge($payload, [
            'platform'      => 'wordpress',
            'host'          => $this->host,
            'versions'      => $installedVersions,
            'php_version'   => PHP_VERSION
        ]);
    
        $response = wp_remote_post($endpoint, [
            'method' => 'POST', 
            'body' => $payload,
            'sslverify' => false,
        ]);
        
        return $response;
    }   

    public function installedVersions()
    {
        $addons = \JReviewsDeploy\jr_addons_array();

        $jreviews = \JReviewsDeploy\plugin_version('jreviews');

        $framework = \JReviewsDeploy\plugin_version('s2framework');
        
        $jreviews = $jreviews == '[VERSION]' ? \JReviewsDeploy\jr_config('jreviews-version') : $jreviews;

        $framework = $framework == '[VERSION]' ? \JReviewsDeploy\jr_config('s2framework-version') : $framework;
        
        return [
            's2framework'   => $framework,
            'jreviews'      => $jreviews,
        ] + $addons;        
    }    
}